#include "inputwidget.h"
#include <qpainter.h>
#include <qapplication.h>
#include <qpe/config.h>
#include <qpe/global.h>

InputWidget::InputWidget(QWidget* parent, const char* name, WFlags f  )
  :QWidget( parent, name, f ), _width(0), _selected( -1 )
{
  reread();
}

void InputWidget::paintEvent( QPaintEvent* )
{
  paint( false, false );
  bitBlt( this, 0, 0, &_buffer );
}

QSize InputWidget::sizeHint() const
{
  paint( false, false );
  return _buffer.size();
}

void InputWidget::paint( bool force, bool reload ) const
{
  int w = qApp->desktop()->width();
  if ( w == _width && ! force )
    return;

  ASSERT( w != 0 );  
  _width = w;
  _buffer.resize( _width, 300 );
  _buffer.fill( backgroundColor() );

  if ( reload )
    _regions.clear();

  QPainter p;
  p.begin(&_buffer);

  // Set font
  QFont fnt = font();
  fnt.setPointSize( _fontSize );
  p.setFont( fnt );

  QFontMetrics fMetrics( fnt );
  int xSpace = 2;
  int ySpace = 2;
  int xOffset = 0;
  int yOffset = 0;
  int charHeight = fMetrics.height();
  int lineHeight = charHeight + 2 * ySpace;
  int descent = fMetrics.descent();

  for ( unsigned int i =0; i < _symbols.length(); ++i ) {
    int charWidth = fMetrics.width( _symbols[i] );
    if ( xOffset + charWidth + 2* xSpace > _width ) {
      xOffset = 0;
      yOffset += lineHeight;
    }

    // Draw or Undraw selection
    QRect rect( xOffset, yOffset, charWidth + 2* xSpace, lineHeight );
    QColor col;
    if ( _selected == (int) i )
      col = gray;
    else
      col = backgroundColor();
    p.setBrush( col );
    p.setPen( NoPen );
    p.drawRect( rect );      

    // Draw text
    p.setPen( black );
    p.drawText( xOffset + xSpace, yOffset + ySpace + charHeight-descent-1, _symbols[i]);

    if ( reload )
      _regions.append( rect );

    xOffset+= charWidth + 2* xSpace;
  }
  p.end();

  QPixmap backup( _buffer );
  _buffer.resize( _width, yOffset+lineHeight );
  bitBlt( &_buffer, 0, 0, &backup );
}

void InputWidget::mousePressEvent( QMouseEvent* event )
{
  QValueList<QRect>::ConstIterator it = _regions.begin();
  int i = 0;
  for ( ; it != _regions.end(); ++it, ++i ) {
    if ( (*it).contains( event->pos() ) ) {
      emit key(_symbols[i].unicode(), 0, 0, true, false);
      _selected = i;
      paint( true, false );
      repaint();
      break;
    }
  }
}

void InputWidget::mouseReleaseEvent( QMouseEvent* )
{
  emit key(_symbols[_selected].unicode(), 0, 0, false, false);
  _selected = -1;
  paint( true, false );
  repaint();
}

void InputWidget::message( const QCString& msg )
{
  if ( msg == "reread()" )
    reread();
}


void InputWidget::reread() 
{
  Config config( "CustomInput" );
  config.setGroup( "CustomInput" );
  _symbols = config.readEntry("symbols", "\\|" );
  _fontSize = config.readNumEntry("fontSize", 12 );
  paint( true, true );
  repaint();
  if ( isVisible() ) {
    Global::hideInputMethod ();
    Global::showInputMethod();
  }
}
